@@ -1,4 +1,4 @@
-# Copyright (c) 2022, DjaoDjin inc.
+# Copyright (c) 2023, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,7 +26,7 @@ import logging
 
 from dateutil.relativedelta import relativedelta
 from django.db import transaction
-from django.db.models import Q
+from django.db.models import Max, Q
 from rest_framework import generics, status
 from rest_framework.exceptions import ValidationError
 from rest_framework.response import Response as HttpResponse
@@ -35,10 +35,10 @@ from .. import settings, signals
 from ..compat import six, gettext_lazy as _
 from ..docs import OpenAPIResponse, swagger_auto_schema
 from ..mixins import AccountMixin
-from ..models import PortfolioDoubleOptIn
-from .serializers import (NoModelSerializer, PortfolioOptInSerializer,
-    PortfolioOptInUpdateSerializer, PortfolioGrantCreateSerializer,
-    PortfolioRequestCreateSerializer)
+from ..models import Portfolio, PortfolioDoubleOptIn, Sample
+from .serializers import (NoModelSerializer, PortfolioReceivedSerializer,
+    PortfolioOptInSerializer, PortfolioOptInUpdateSerializer,
+    PortfolioGrantCreateSerializer, PortfolioRequestCreateSerializer)
 from ..filters import (DateRangeFilter, DoubleOptInStateFilter, OrderingFilter,
      SearchFilter)
 from ..utils import datetime_or_now, get_account_model
@@ -49,8 +49,6 @@ LOGGER = logging.getLogger(__name__)
 
 class SmartPortfolioListMixin(AccountMixin):
 
-    serializer_class = PortfolioOptInSerializer
-
     search_fields = (
         'grantee__full_name',
         'account__full_name',
@@ -100,7 +98,11 @@ class PortfoliosAPIView(SmartPortfolioListMixin, generics.ListAPIView):
               {
                 "grantee": "energy-utility",
                 "account": "supplier-1",
-                "campaign": "sustainability",
+                "campaign": {
+                    "slug": "sustainability",
+                    "title": "ESG/Environmental practices",
+                    "account": "tspproject"
+                },
                 "ends_at": "2022-01-01T00:00:00Z",
                 "state": "request-initiated",
                 "api_accept": "/api/supplier-1/portfolios/requests/\
@@ -109,13 +111,58 @@ class PortfoliosAPIView(SmartPortfolioListMixin, generics.ListAPIView):
             ]
         }
     """
+    serializer_class = PortfolioReceivedSerializer
+
+    def decorate_queryset(self, queryset):
+        latest_frozen_by_campaigns = {}
+        latest_shared_by_campaigns = {}
+        for optin in queryset:
+            campaign = optin.campaign
+            query_kwargs = {'campaign': campaign} if campaign else {}
+            optin.expected_behavior = optin.EXPECTED_CREATE
+            if campaign:
+                latest_frozen_at = latest_frozen_by_campaigns.get(campaign)
+                if (not latest_frozen_at and
+                    campaign not in latest_frozen_by_campaigns):
+                    latest_frozen_at = Sample.objects.filter(
+                        account=self.account, is_frozen=True,
+                        **query_kwargs).aggregate(Max('created_at'))
+                    if latest_frozen_at:
+                        latest_frozen_at = latest_frozen_at.get(
+                            'created_at__max')
+                    latest_frozen_by_campaigns.update({
+                        campaign: latest_frozen_at})
+
+                latest_shared_at = latest_shared_by_campaigns.get(campaign)
+                if (not latest_shared_at and
+                    campaign not in latest_shared_by_campaigns):
+                    latest_shared_at = Portfolio.objects.filter(
+                        account=self.account, grantee=optin.grantee,
+                        **query_kwargs).aggregate(Max('ends_at'))
+                    if latest_shared_at:
+                        latest_shared_at = latest_shared_at.get('ends_at__max')
+                    latest_shared_by_campaigns.update({
+                        campaign: latest_shared_at})
+
+                # compute expected behavior
+                if not latest_frozen_at:
+                    optin.expected_behavior = optin.EXPECTED_CREATE
+                elif (latest_shared_at and
+                      latest_frozen_at < latest_shared_at and
+                      latest_shared_at < optin.created_at):
+                    optin.expected_behavior = optin.EXPECTED_UPDATE
+                else:
+                    optin.expected_behavior = optin.EXPECTED_SHARE
+
+        return queryset
+
 
     def get_queryset(self):
-        return PortfolioDoubleOptIn.objects.filter(
-            (Q(account=self.account) &
-            Q(state=PortfolioDoubleOptIn.OPTIN_REQUEST_INITIATED)) |
-            (Q(grantee=self.account) &
-            Q(state=PortfolioDoubleOptIn.OPTIN_GRANT_INITIATED)))
+        return PortfolioDoubleOptIn.objects.pending_for(self.account)
+
+    def paginate_queryset(self, queryset):
+        page = super(PortfoliosAPIView, self).paginate_queryset(queryset)
+        return self.decorate_queryset(page if page else queryset)
 
 
 class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
@@ -145,7 +192,11 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
                 {
                   "grantee": "water-utility",
                   "account": "supplier-1",
-                  "campaign": "sustainability",
+                  "campaign": {
+                      "slug": "sustainability",
+                      "title": "ESG/Environmental practices",
+                      "account": "tspproject"
+                  },
                   "ends_at": "2022-01-01T00:00:00Z",
                   "state": "grant-initiated",
                   "api_accept": "/api/water-utility/portfolios/grants\
@@ -154,8 +205,8 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
             ]
         }
     """
-    serializer_class = PortfolioOptInSerializer
     lookup_field = settings.ACCOUNT_LOOKUP_FIELD
+    serializer_class = PortfolioOptInSerializer
 
     def get_queryset(self):
         return PortfolioDoubleOptIn.objects.filter(
@@ -202,7 +253,7 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
                "results": [{
                  "grantee": "water-utility",
                  "account": "supplier-1",
-                 "campaign": "sustainability",
+                 "campaign": null,
                  "ends_at": "2022-01-01T00:00:00Z",
                  "state": "grant-initiated",
                  "api_accept": "/api/water-utility/portfolios/grants\
@@ -348,6 +399,8 @@ class PortfoliosGrantAcceptAPIView(AccountMixin, generics.DestroyAPIView):
     serializer_class = NoModelSerializer
 
     def get_queryset(self):
+        # Look up grant to be accepted through the 'verification_key'
+        # so it OK to just filter by grantee.
         return PortfolioDoubleOptIn.objects.filter(grantee=self.account)
 
     def post(self, request, *args, **kwargs):
@@ -422,7 +475,11 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
               {
                 "grantee": "energy-utility",
                 "account": "supplier-1",
-                "campaign": "sustainability",
+                "campaign": {
+                    "slug": "sustainability",
+                    "title": "ESG/Environmental practices",
+                    "account": "tspproject"
+                },
                 "ends_at": "2022-01-01T00:00:00Z",
                 "state": "request-denied",
                 "api_accept": null
@@ -430,7 +487,11 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
               {
                 "grantee": "energy-utility",
                 "account": "supplier-1",
-                "campaign": "sustainability",
+                "campaign": {
+                    "slug": "sustainability",
+                    "title": "ESG/Environmental practices",
+                    "account": "tspproject"
+                },
                 "ends_at": "2022-01-01T00:00:00Z",
                 "state": "request-initiated",
                 "api_accept": "/api/supplier-1/portfolios/requests/\
@@ -439,7 +500,11 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
               {
                 "grantee": "energy-utility",
                 "account": "andy-shop",
-                "campaign": "sustainability",
+                "campaign": {
+                    "slug": "sustainability",
+                    "title": "ESG/Environmental practices",
+                    "account": "tspproject"
+                },
                 "ends_at": "2022-01-01T00:00:00Z",
                 "state": "request-initiated",
                 "api_accept": "/api/andy-shop/portfolios/requests/\
@@ -452,6 +517,8 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
     serializer_class = PortfolioOptInSerializer
 
     def get_queryset(self):
+        # Look up request to be accepted through the 'verification_key'
+        # so it OK to just filter by grantee.
         return PortfolioDoubleOptIn.objects.filter(grantee=self.account)
 
     def post(self, request, *args, **kwargs):
@@ -504,7 +571,6 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
         ends_at = datetime_or_now()
         defaults = {
             'state': PortfolioDoubleOptIn.OPTIN_REQUEST_INITIATED,
-            'ends_at': ends_at,
             'initiated_by': self.request.user
         }
         accounts = serializer.validated_data['accounts']
@@ -552,6 +618,8 @@ class PortfoliosRequestAcceptAPIView(AccountMixin, generics.DestroyAPIView):
     serializer_class = NoModelSerializer
 
     def get_queryset(self):
+        # Look up request to be accepted through the 'verification_key'
+        # so it OK to just filter by account.
         return PortfolioDoubleOptIn.objects.filter(account=self.account)
 
     def post(self, request, *args, **kwargs):
@@ -655,7 +723,11 @@ class PortfoliosUpdateAPIView(AccountMixin, generics.UpdateAPIView):
         {
           "grantee": "energy-utility",
           "account": "supplier-1",
-          "campaign": "sustainability",
+          "campaign": {
+              "slug": "sustainability",
+              "title": "ESG/Environmental practices",
+              "account": "tspproject"
+          },
           "ends_at": "2022-01-01T00:00:00Z",
           "state": "request-denied",
           "api_accept": null,
