@@ -40,7 +40,7 @@ from ..compat import six, is_authenticated
 from ..docs import OpenAPIResponse, swagger_auto_schema
 from ..filters import DateRangeFilter, OrderingFilter, SampleStateFilter
 from ..mixins import AccountMixin, SampleMixin
-from ..models import Answer, Choice, Sample, Unit, UnitEquivalences
+from ..models import Answer, Choice, Portfolio, Sample, Unit, UnitEquivalences
 from ..queries import is_sqlite3
 from ..utils import datetime_or_now, get_question_model, get_user_serializer
 from .serializers import (AnswerSerializer, NoModelSerializer,
@@ -311,7 +311,10 @@ class SampleAPIView(SampleMixin, generics.RetrieveAPIView):
         "updated_at": "2018-01-24T17:03:34.926193Z",
         "is_frozen": false,
         "account": "supplier-1",
-        "campaign": "sustainability"
+        "campaign": {
+            "slug": "sustainability",
+            "title": "ESG/Environmental practices"
+        }
     }
     """
     serializer_class = SampleSerializer
@@ -424,7 +427,8 @@ SELECT
     answers.sample_id AS sample_id,
     questions.rank AS _rank,
     questions.required AS required,
-    answers._measured_text AS _measured_text
+    answers._measured_text AS _measured_text,
+    1 AS frozen
 FROM questions
 LEFT OUTER JOIN answers
   ON questions.id = answers.question_id""" % {
@@ -432,9 +436,9 @@ LEFT OUTER JOIN answers
       'convert_to_text': ("" if is_sqlite3() else "::text"),
       'extra_question_clause': extra_question_clause,
       'prefix': prefix,
-      'sample': sample.pk,
+      'sample': sample.pk
   }
-        else:
+        else: # `not sample.is_frozen`
             query_text = """
 WITH answers AS (
     SELECT
@@ -477,7 +481,8 @@ SELECT
     answers.sample_id AS sample_id,
     campaign_questions.rank AS _rank,
     campaign_questions.required AS required,
-    answers._measured_text AS _measured_text
+    answers._measured_text AS _measured_text,
+    0 AS frozen
 FROM campaign_questions
 LEFT OUTER JOIN answers
   ON campaign_questions.id = answers.question_id""" % {
@@ -735,7 +740,8 @@ answers/construction HTTP/1.1
         errors = []
 
         if self.sample.is_frozen:
-            raise ValidationError("sample is frozen")
+            raise ValidationError({
+                'detail': "cannot update answers in a frozen sample"})
 
         for datapoint in validated_data:
             measured = datapoint.get('measured', None)
@@ -1060,7 +1066,8 @@ class SampleCandidatesAPIView(SampleCandidatesMixin, SampleAnswersMixin,
     def create(self, request, *args, **kwargs):
         #pylint:disable=too-many-locals
         if self.sample.is_frozen:
-            raise ValidationError("sample is frozen")
+            raise ValidationError({
+                'detail': "cannot update answers in a frozen sample"})
 
         at_time = datetime_or_now()
         at_least_one_created = False
@@ -1364,6 +1371,8 @@ class SampleFreezeAPIView(SampleMixin, generics.CreateAPIView):
     serializer_class = SampleSerializer
 
     def create(self, request, *args, **kwargs):
+        if self.sample.is_frozen:
+            raise ValidationError({'detail': "sample is already frozen"})
         self.sample.is_frozen = True
         self.sample.save()
         serializer = self.get_serializer(self.sample)
@@ -1404,6 +1413,9 @@ class SampleResetAPIView(SampleMixin, generics.CreateAPIView):
     serializer_class = SampleSerializer
 
     def create(self, request, *args, **kwargs):
+        if self.sample.is_frozen:
+            raise ValidationError({
+                'detail': "cannot update answers in a frozen sample"})
         prefix = self.path
         if prefix:
             queryset = self.sample.answers.filter(
@@ -1454,9 +1466,9 @@ class SampleResetIndexAPIView(SampleResetAPIView):
 
 class SampleRecentCreateAPIView(AccountMixin, generics.ListCreateAPIView):
     """
-    Lists updatable samples
+    Lists historical samples
 
-    This API end-point returns all samples which have not been frozen
+    This API end-point returns all samples which have or have not been frozen
     for an account.
 
     **Tags**: assessments
@@ -1505,12 +1517,44 @@ class SampleRecentCreateAPIView(AccountMixin, generics.ListCreateAPIView):
 
     serializer_class = SampleSerializer
 
+    def decorate_queryset(self, queryset):
+        frozen_by_campaigns = {}
+        for sample in queryset:
+            if sample.is_frozen:
+                if sample.campaign not in frozen_by_campaigns:
+                    frozen_by_campaigns[sample.campaign] = [sample]
+                else:
+                    frozen_by_campaigns[sample.campaign] += [sample]
+        for campaign, samples in six.iteritems(frozen_by_campaigns):
+            next_level = Portfolio.objects.filter(
+                account=self.account, campaign=campaign).values(
+                    'ends_at', 'grantee__slug').order_by('-ends_at')
+            for sample in sorted(samples,
+                        key=lambda smp: smp.created_at, reverse=True):
+                sample.grantees = []
+                level = next_level
+                next_level = []
+                for accessible in level:
+                    ends_at = accessible.get('ends_at')
+                    grantee = accessible.get('grantee__slug')
+                    if sample.created_at <= ends_at:
+                        sample.grantees += [grantee]
+                    else:
+                        next_level += [accessible]
+        return queryset
+
+
     def get_queryset(self):
         return Sample.objects.filter(
             account=self.account,
             extra__isnull=True         # XXX convinience
         ).select_related('campaign')
 
+    def paginate_queryset(self, queryset):
+        page = super(
+            SampleRecentCreateAPIView, self).paginate_queryset(queryset)
+        return self.decorate_queryset(page if page else queryset)
+
     @swagger_auto_schema(request_body=SampleCreateSerializer)
     def post(self, request, *args, **kwargs):
         """
