@@ -209,9 +209,13 @@ class QuestionCreateSerializer(serializers.ModelSerializer):
 
 class QuestionDetailSerializer(QuestionCreateSerializer):
 
+    title = serializers.CharField(required=False,
+        help_text=_("Title of the question as displayed in user interfaces"))
+
     class Meta:
         model = QuestionCreateSerializer.Meta.model
         fields = QuestionCreateSerializer.Meta.fields + ('path',)
+        read_only_fields = ('path',)
 
 
 class QuestionSerializer(serializers.ModelSerializer):
@@ -254,7 +258,7 @@ class CampaignSerializer(serializers.ModelSerializer):
     account = serializers.SlugRelatedField(
         slug_field=settings.BELONGS_LOOKUP_FIELD,
         queryset=get_belongs_model().objects.all(),
-        help_text=("Account this sample belongs to."))
+        help_text=_("Account this sample belongs to."))
 
     class Meta(object):
         model = Campaign
@@ -278,7 +282,7 @@ class CampaignDetailSerializer(serializers.ModelSerializer):
     account = serializers.SlugRelatedField(
         slug_field=settings.BELONGS_LOOKUP_FIELD,
         queryset=get_belongs_model().objects.all(),
-        help_text=("Account this sample belongs to."))
+        help_text=_("Account this sample belongs to."))
     questions = QuestionSerializer(many=True)
 
     class Meta(object):
@@ -292,7 +296,7 @@ class SampleCreateSerializer(serializers.ModelSerializer):
 
     campaign = serializers.SlugRelatedField(slug_field='slug',
         queryset=Campaign.objects.all(), required=False,
-        help_text=("Campaign this sample is part of."))
+        help_text=_("Campaign this sample is part of."))
 
     class Meta(object):
         model = Sample
@@ -303,19 +307,21 @@ class SampleSerializer(SampleCreateSerializer):
 
     campaign = CampaignSerializer(
         read_only=True, allow_null=True,
-        help_text=("Campaign this sample is part of."))
+        help_text=_("Campaign this sample is part of"))
     account = serializers.SlugRelatedField(slug_field='slug',
         read_only=True, required=False,
-        help_text=("Account this sample belongs to."))
+        help_text=_("Account this sample belongs to"))
     location = serializers.URLField(read_only=True, allow_null=True,
-        help_text=("URL at which the response is visible."))
+        help_text=_("URL at which the response is visible"))
+    grantees = serializers.ListField(required=False,
+        help_text=_("Profiles with which sample was shared"))
 
     class Meta(object):
         model = Sample
         fields = ('campaign', 'slug', 'account', 'created_at',
-            'updated_at', 'is_frozen', 'location')
+            'updated_at', 'is_frozen', 'location', 'grantees')
         read_only_fields = ('campaign', 'slug', 'account', 'created_at',
-            'updated_at', 'is_frozen', 'location')
+            'updated_at', 'is_frozen', 'location', 'grantees')
 
     @staticmethod
     def get_location(obj):
@@ -469,9 +475,12 @@ class MatrixSerializer(serializers.ModelSerializer):
 
 
 class KeyValueTuple(serializers.ListField):
+    # `KeyValueTuple` is typed as a (String, Integer) tuple.
+    # by not specifying a child field, the serialized data
+    # is generated as expected. Otherwise we would end up
+    # with a (String, String).
 
-    child = serializers.CharField(allow_null=True) # XXX (String, Integer)
-    min_length = 3
+    min_length = 2
     max_length = 3
 
 
@@ -479,9 +488,9 @@ class TableSerializer(NoModelSerializer):
 
     slug = serializers.SlugField(
         help_text=_("Unique key in the table for the data series"))
-    printable_name = serializers.CharField(
-        required=False, read_only=True,
-        help_text=_("Name that can be safely used for display in HTML pages"))
+    title = serializers.CharField(required=False, read_only=True,
+        help_text=_("Title of data serie that can be safely used for display"\
+        " in HTML pages"))
     extra = ExtraField(required=False,
         help_text=_("Extra meta data (can be stringify JSON)"))
     values = serializers.ListField(
@@ -509,7 +518,8 @@ class MetricsSerializer(NoModelSerializer):
         help_text=_("Unit the measured field is in"))
     title = serializers.CharField(
         help_text=_("Title for the table"))
-    results = TableSerializer(many=True)
+    results = TableSerializer(many=True,
+        help_text=_("Data series"))
 
 
 class CompareQuestionSerializer(QuestionSerializer):
@@ -581,7 +591,10 @@ class PortfolioOptInUpdateSerializer(serializers.ModelSerializer):
         fields = ('extra',)
 
 
-class PortfolioOptInSerializer(serializers.ModelSerializer):
+class PortfolioReceivedSerializer(serializers.ModelSerializer):
+
+    # `PortfolioReceivedSerializer` is used in `PortfoliosAPIView` which
+    # is used by the receiving profile.
 
     grantee = serializers.SlugRelatedField(
         queryset=get_account_model().objects.all(),
@@ -589,20 +602,20 @@ class PortfolioOptInSerializer(serializers.ModelSerializer):
     account = serializers.SlugRelatedField(
         queryset=get_account_model().objects.all(),
         slug_field=settings.ACCOUNT_LOOKUP_FIELD)
-    campaign = serializers.SlugRelatedField(
-        queryset=Campaign.objects.all(), slug_field='slug')
+    campaign = CampaignSerializer(allow_null=True,
+        help_text=_("Campaign granted/requested"))
     state = EnumField(choices=PortfolioDoubleOptIn.STATES)
+    expected_behavior = EnumField(
+        choices=PortfolioDoubleOptIn.EXPECTED_BEHAVIOR, required=False)
     api_accept = serializers.SerializerMethodField()
     api_remove = serializers.SerializerMethodField()
-    extra = ExtraField(required=False,
-        help_text=_("Extra meta data (can be stringify JSON)"))
 
     class Meta:
         model = PortfolioDoubleOptIn
         fields = ('grantee', 'account', 'campaign', 'created_at', 'ends_at',
-            'state', 'api_accept', 'api_remove', 'extra')
+            'state', 'expected_behavior', 'api_accept', 'api_remove')
         read_only_fields = ('created_at', 'ends_at',
-            'state', 'api_accept', 'api_remove')
+            'state', 'expected_behavior', 'api_accept', 'api_remove')
 
     def get_api_accept(self, obj):
         api_endpoint = None
@@ -637,6 +650,18 @@ class PortfolioOptInSerializer(serializers.ModelSerializer):
         return api_endpoint
 
 
+class PortfolioOptInSerializer(PortfolioReceivedSerializer):
+
+    extra = ExtraField(required=False,
+        help_text=_("Extra meta data (can be stringify JSON)"))
+
+    class Meta(PortfolioReceivedSerializer.Meta):
+        fields = PortfolioReceivedSerializer.Meta.fields + (
+            'extra',)
+        read_only_fields = PortfolioReceivedSerializer.Meta.read_only_fields + (
+            'extra',)
+
+
 class AccountsFilterAddSerializer(serializers.ModelSerializer):
 
     slug = serializers.CharField(required=False)
