@@ -42,11 +42,40 @@ from .utils import datetime_or_now
 LOGGER = logging.getLogger(__name__)
 
 
+def search_terms_as_list(params):
+    """
+    Search terms are set by a ?q=... query parameter.
+    When multiple search terms must be matched, they can be delimited
+    by a comma.
+    """
+    params = params.replace('\x00', '')  # strip null characters
+    results = []
+    inside = False
+    first = 0
+    for last, letter in enumerate(params):
+        if inside:
+            if letter == '"':
+                if first < last:
+                    results += [params[first:last]]
+                first = last + 1
+                inside = False
+        else:
+            if letter in (',',):
+                if first < last:
+                    results += [params[first:last]]
+                first = last + 1
+            elif letter == '"':
+                inside = True
+                first = last + 1
+    if first < len(params):
+        results += [params[first:len(params)]]
+    return results
+
+
 class SearchFilter(BaseSearchFilter):
 
     search_field_param = settings.SEARCH_FIELDS_PARAM
 
-
     def filter_queryset(self, request, queryset, view):
         search_fields = self.get_valid_fields(request, queryset, view)
         search_terms = self.get_search_terms(request)
@@ -80,8 +109,7 @@ class SearchFilter(BaseSearchFilter):
         return queryset
 
 
-    @staticmethod
-    def filter_valid_fields(queryset, fields, view):
+    def filter_valid_fields(self, queryset, fields, view):
         #pylint:disable=protected-access
         model_fields = {
             field.name for field in queryset.model._meta.get_fields()}
@@ -89,8 +117,14 @@ class SearchFilter(BaseSearchFilter):
         # which are not present in the model.
         alternate_fields = getattr(view, 'alternate_fields', {})
         for field in fields:
+            field_lookup = field[0]
+            lookup = self.lookup_prefixes.get(field_lookup)
+            if lookup:
+                field = field[1:]
             alternate_field = alternate_fields.get(field, None)
             if alternate_field:
+                if lookup:
+                    alternate_field = field_lookup + alternate_field
                 if isinstance(alternate_field, (list, tuple)):
                     fields += tuple(alternate_field)
                 else:
@@ -98,8 +132,12 @@ class SearchFilter(BaseSearchFilter):
 
         valid_fields = []
         for field in fields:
-            if '__' in field:
-                relation, rel_field = field.split('__')
+            field_name = field
+            lookup = self.lookup_prefixes.get(field_name[0])
+            if lookup:
+                field_name = field_name[1:]
+            if '__' in field_name:
+                relation, rel_field = field_name.split('__')
                 try:
                     # check if the field is a relation
                     rel = queryset.model._meta.get_field(relation).remote_field
@@ -110,8 +148,8 @@ class SearchFilter(BaseSearchFilter):
                         valid_fields.append(field)
                 except FieldDoesNotExist:
                     pass
-            elif field in model_fields:
-                rel = queryset.model._meta.get_field(field).remote_field
+            elif field_name in model_fields:
+                rel = queryset.model._meta.get_field(field_name).remote_field
                 if not rel:
                     # if it is a relation fields (as a result valid),
                     # we don't want to end-up with a problem later on
@@ -121,8 +159,24 @@ class SearchFilter(BaseSearchFilter):
         return tuple(valid_fields)
 
 
+    def get_query_param(self, request, key, default_value=None):
+        try:
+            return request.query_params.get(key, default_value)
+        except AttributeError:
+            pass
+        return request.GET.get(key, default_value)
+
+
+    def get_query_param_list(self, request, key, default_value=None):
+        try:
+            return request.query_params.getlist(key, default_value)
+        except AttributeError:
+            pass
+        return request.GET.getlist(key, default_value)
+
+
     def get_query_fields(self, request):
-        return request.query_params.getlist(self.search_field_param)
+        return self.get_query_param_list(request, self.search_field_param)
 
 
     def get_search_terms(self, request):
@@ -131,29 +185,8 @@ class SearchFilter(BaseSearchFilter):
         When multiple search terms must be matched, they can be delimited
         by a comma.
         """
-        params = request.query_params.get(self.search_param, '')
-        params = params.replace('\x00', '')  # strip null characters
-        results = []
-        inside = False
-        first = 0
-        for last, letter in enumerate(params):
-            if inside:
-                if letter == '"':
-                    if first < last:
-                        results += [params[first:last]]
-                    first = last + 1
-                    inside = False
-            else:
-                if letter in (',',):
-                    if first < last:
-                        results += [params[first:last]]
-                    first = last + 1
-                elif letter == '"':
-                    inside = True
-                    first = last + 1
-        if first < len(params):
-            results += [params[first:len(params)]]
-        return results
+        return search_terms_as_list(
+            self.get_query_param(request, self.search_param, ''))
 
 
     def get_valid_fields(self, request, queryset, view, context=None):
@@ -212,6 +245,13 @@ class SearchFilter(BaseSearchFilter):
 
 class OrderingFilter(BaseOrderingFilter):
 
+    def get_query_param(self, request, key, default_value=None):
+        try:
+            return request.query_params.getlist(key, default_value)
+        except AttributeError:
+            pass
+        return request.GET.get(key, default_value)
+
     def get_valid_fields(self, queryset, view, context=None):
         #pylint:disable=protected-access
         if context is None:
@@ -276,7 +316,7 @@ class OrderingFilter(BaseOrderingFilter):
             ordering = self.remove_invalid_fields(
                 queryset, default, view, request)
         default_ordering = list(ordering)
-        params = request.query_params.getlist(self.ordering_param)
+        params = self.get_query_param(request, self.ordering_param)
         if params:
             if isinstance(params, six.string_types):
                 params = params.split(',')
@@ -407,8 +447,15 @@ class StateFilter(BaseFilterBackend):
     state_field = 'state'
     state_candidates = []
 
+    def get_query_param(self, request, key, default_value=None):
+        try:
+            return request.query_params.get(key, default_value)
+        except AttributeError:
+            pass
+        return request.GET.get(key, default_value)
+
     def filter_queryset(self, request, queryset, view):
-        state = request.query_params.get(self.state_param, '')
+        state = self.get_query_param(request, self.state_param, '')
         flt = None
         for val in state.split(','):
             for candidate in self.state_candidates:
