@@ -776,6 +776,25 @@ class PortfolioDoubleOptInQuerySet(models.QuerySet):
             kwargs.update({'ends_at__gte': until})
         return self.filter(grantee=account, **kwargs)
 
+
+    def pending_for(self, account, campaign=None, at_time=None):
+        """
+        Returns the current portfolio grant/request for `account`
+        that a user must either accept or deny.
+        """
+        if not at_time:
+            at_time = datetime_or_now()
+        kwargs = {}
+        if campaign:
+            kwargs.update({'campaign': campaign})
+        return self.filter(
+            (models.Q(ends_at__isnull=True) | models.Q(ends_at__gte=at_time)) &
+            (models.Q(account=account) &
+            models.Q(state=PortfolioDoubleOptIn.OPTIN_REQUEST_INITIATED)) |
+            (models.Q(grantee=account) &
+            models.Q(state=PortfolioDoubleOptIn.OPTIN_GRANT_INITIATED)),
+            **kwargs)
+
     def requested(self, account, campaign=None,
                   start_at=None, ends_at=None, until=None):
         """
@@ -835,6 +854,10 @@ class PortfolioDoubleOptInManager(models.Manager):
         return self.get_queryset().by_grantee(account, campaign=campaign,
             start_at=start_at, ends_at=ends_at, until=until)
 
+    def pending_for(self, account, campaign=None, at_time=None):
+        return self.get_queryset().pending_for(account,
+            campaign=campaign, at_time=at_time)
+
     def requested(self, account, campaign=None,
                   start_at=None, ends_at=None, until=None):
         #pylint:disable=too-many-arguments
@@ -918,6 +941,16 @@ class PortfolioDoubleOptIn(models.Model):
         (OPTIN_REQUEST_EXPIRED, 'request-expired'),
     ]
 
+    EXPECTED_SHARE = 0
+    EXPECTED_CREATE = 1
+    EXPECTED_UPDATE = 2
+
+    EXPECTED_BEHAVIOR = [
+        (EXPECTED_SHARE, 'share'),
+        (EXPECTED_CREATE, 'create'),
+        (EXPECTED_UPDATE, 'update'),
+    ]
+
     objects = PortfolioDoubleOptInManager()
 
     created_at = models.DateTimeField(auto_now_add=True,
@@ -951,12 +984,14 @@ class PortfolioDoubleOptIn(models.Model):
             "campaign='%s', ends_at='%s')>" % (self.grantee_id,
             self.account_id, self.campaign_id, self.ends_at)
 
-    def create_portfolios(self):
+    def create_portfolios(self, at_time=None):
+        if not at_time:
+            at_time = datetime_or_now()
         with transaction.atomic():
             Portfolio.objects.update_or_create(
                 grantee=self.grantee, account=self.account,
                 campaign=self.campaign,
-                defaults={'ends_at': self.ends_at})
+                defaults={'ends_at': at_time})
 
     @staticmethod
     def generate_key(account):
