@@ -35,7 +35,7 @@ from django.apps import apps as django_apps
 from django.conf import settings as django_settings
 from django.contrib.auth import REDIRECT_FIELD_NAME
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import Count, F
+from django.db.models import Count, F, FilteredRelation, Q
 from django.http.request import split_domain_port, validate_host
 from django.utils.timezone import utc
 
@@ -55,8 +55,8 @@ def as_timestamp(dtime_at=None):
         dtime_at - datetime.datetime(1970, 1, 1, tzinfo=utc)).total_seconds())
 
 
-def get_accessible_accounts(grantees,
-                            campaign=None, start_at=None, ends_at=None):
+def get_accessible_accounts(grantees, campaign=None, aggregate_set=False,
+                            start_at=None, ends_at=None):
     """
     All accounts which have elected to share samples with at least one
     account in grantees.
@@ -70,7 +70,8 @@ def get_accessible_accounts(grantees,
     if (hasattr(settings, 'ACCESSIBLE_ACCOUNTS_CALLABLE') and
         settings.ACCESSIBLE_ACCOUNTS_CALLABLE):
         queryset = import_string(settings.ACCESSIBLE_ACCOUNTS_CALLABLE)(
-            grantees, campaign=campaign, start_at=start_at, ends_at=ends_at)
+            grantees, campaign=campaign, aggregate_set=aggregate_set,
+            start_at=start_at, ends_at=ends_at)
 
     if queryset is None:
         filter_params = {}
@@ -79,10 +80,26 @@ def get_accessible_accounts(grantees,
                 'portfolio_double_optin_accounts__created_at__gte': start_at})
         if campaign:
             filter_params.update({'portfolios__campaign': campaign})
+        # Implementation note: requires Django>=2 because of `FilteredRelation`
+        # Adding the correct condition on the LEFT OUTER JOIN is quite
+        # a challenge with Django. The SQL we need is as follow:
+        #   SELECT DISTINCT saas_organization.*,
+        #                 survey_portfoliodoubleoptin.extra AS _extra
+        #   FROM saas_organization
+        #   INNER JOIN survey_portfolio
+        #   ON saas_organization.id = survey_portfolio.account_id
+        #   LEFT OUTER JOIN survey_portfoliodoubleoptin
+        #   ON saas_organization.id = survey_portfoliodoubleoptin.account_id
+        #     AND survey_portfoliodoubleoptin.grantee_id IN (${grantees})
+        #   WHERE survey_portfolio.grantee_id IN (${grantees});
         queryset = get_account_model().objects.filter(
             portfolios__grantee__in=grantees,
             **filter_params).annotate(
-                _extra=F('portfolio_double_optin_accounts__extra')).distinct()
+            granted=FilteredRelation(
+                'portfolio_double_optin_accounts',
+                condition=Q(
+                    portfolio_double_optin_accounts__grantee__in=grantees
+            ))).annotate(_extra=F('granted__extra'))
 
     return queryset
 
@@ -126,13 +143,15 @@ def get_benchmarks_enumerated(samples, questions, questions_by_key=None):
             questions_by_key.update({question_pk: value})
 
     # per-choice number of answers
-    for row in Answer.objects.filter(
+    enum_answers = Answer.objects.filter(
             question__in=questions,
             unit_id=F('question__default_unit_id'),
             sample_id__in=samples,
-            question__default_unit__system=Unit.SYSTEM_ENUMERATED,
+            question__default_unit__system__in=[Unit.SYSTEM_ENUMERATED,
+                Unit.SYSTEM_DATETIME], # XXX target year are stored as choices
             unit__enums__id=F('measured')).values('question__id',
-                'measured', 'unit__enums__text').annotate(Count('sample_id')):
+                'unit__enums__text').annotate(Count('sample_id'))
+    for row in enum_answers:
         question_pk = row['question__id']
         count = row['sample_id__count']
         measured = row['unit__enums__text']
